@@ -38,6 +38,7 @@ from tabulate import tabulate
 
 import pcluster.utils as utils
 from pcluster.config.pcluster_config import PclusterConfig
+from pcluster.constants import PCLUSTER_STACK_PREFIX
 
 if sys.version_info[0] >= 3:
     from urllib.request import urlretrieve
@@ -282,7 +283,7 @@ def update(args):  # noqa: C901 FIXME!!!
         if not args.nowait:
             while stack_status == "UPDATE_IN_PROGRESS":
                 stack_status = utils.get_stack(stack_name, cfn).get("StackStatus")
-                events = cfn.describe_stack_events(StackName=stack_name).get("StackEvents")[0]
+                events = utils.get_stack_events(stack_name)[0]
                 resource_status = (
                     "Status: %s - %s" % (events.get("LogicalResourceId"), events.get("ResourceStatus"))
                 ).ljust(80)
@@ -349,13 +350,8 @@ def _get_batch_ce(stack_name):
     :param config: config
     :return: ce_name or exit if not found
     """
-    cfn = boto3.client("cloudformation")
-    try:
-        outputs = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get("Outputs")
-        return utils.get_stack_output_value(outputs, "BatchComputeEnvironmentArn")
-    except ClientError as e:
-        LOGGER.critical(e.response.get("Error").get("Message"))
-        sys.exit(1)
+    outputs = utils.get_stack(stack_name).get("Outputs")
+    return utils.get_stack_output_value(outputs, "BatchComputeEnvironmentArn")
 
 
 def _get_pcluster_version_from_stack(stack):
@@ -392,11 +388,11 @@ def list_stacks(args):
     try:
         result = []
         for stack in utils.paginate_boto3(boto3.client("cloudformation").describe_stacks):
-            if stack.get("ParentId") is None and stack.get("StackName").startswith(utils.PCLUSTER_STACK_PREFIX):
+            if stack.get("ParentId") is None and stack.get("StackName").startswith(PCLUSTER_STACK_PREFIX):
                 pcluster_version = _get_pcluster_version_from_stack(stack)
                 result.append(
                     [
-                        stack.get("StackName")[len(utils.PCLUSTER_STACK_PREFIX) :],  # noqa: E203
+                        stack.get("StackName")[len(PCLUSTER_STACK_PREFIX) :],  # noqa: E203
                         _colorize(stack.get("StackStatus"), args),
                         pcluster_version,
                     ]
@@ -447,13 +443,7 @@ def _poll_master_server_state(stack_name):
 
 
 def _get_ec2_instances(stack):
-    try:
-        resources = boto3.client("cloudformation").describe_stack_resources(StackName=stack).get("StackResources")
-    except ClientError as e:
-        LOGGER.critical(e.response.get("Error").get("Message"))
-        sys.stdout.flush()
-        sys.exit(1)
-
+    resources = utils.get_stack_resources(stack)
     temp_instances = [r for r in resources if r.get("ResourceType") == "AWS::EC2::Instance"]
 
     stack_instances = []
@@ -465,12 +455,8 @@ def _get_ec2_instances(stack):
 
 def _get_asg_name(stack_name):
     try:
-        resources = boto3.client("cloudformation").describe_stack_resources(StackName=stack_name).get("StackResources")
+        resources = utils.get_stack_resources(stack_name)
         return [r for r in resources if r.get("LogicalResourceId") == "ComputeFleet"][0].get("PhysicalResourceId")
-    except ClientError as e:
-        LOGGER.critical(e.response.get("Error").get("Message"))
-        sys.stdout.flush()
-        sys.exit(1)
     except IndexError:
         LOGGER.critical("Stack %s does not have a ComputeFleet", stack_name)
         sys.exit(1)
@@ -584,11 +570,11 @@ def status(args):  # noqa: C901 FIXME!!!
 
     cfn = boto3.client("cloudformation")
     try:
-        stack_status = utils.get_stack(stack_name, cfn).get("StackStatus")
-        sys.stdout.write("\rStatus: %s" % stack_status)
+        stack = utils.get_stack(stack_name, cfn)
+        sys.stdout.write("\rStatus: %s" % stack.get("StackStatus"))
         sys.stdout.flush()
         if not args.nowait:
-            while stack_status not in [
+            while stack.get("StackStatus") not in [
                 "CREATE_COMPLETE",
                 "UPDATE_COMPLETE",
                 "UPDATE_ROLLBACK_COMPLETE",
@@ -597,22 +583,26 @@ def status(args):  # noqa: C901 FIXME!!!
                 "DELETE_FAILED",
             ]:
                 time.sleep(5)
-                stack_status = utils.get_stack(stack_name, cfn).get("StackStatus")
-                events = cfn.describe_stack_events(StackName=stack_name).get("StackEvents")[0]
+                stack = utils.get_stack(stack_name, cfn)
+                events = utils.get_stack_events(stack_name)[0]
                 resource_status = (
                     "Status: %s - %s" % (events.get("LogicalResourceId"), events.get("ResourceStatus"))
                 ).ljust(80)
                 sys.stdout.write("\r%s" % resource_status)
                 sys.stdout.flush()
-            sys.stdout.write("\rStatus: %s\n" % stack_status)
+            sys.stdout.write("\rStatus: %s\n" % stack.get("StackStatus"))
             sys.stdout.flush()
-            if stack_status in ["CREATE_COMPLETE", "UPDATE_COMPLETE"]:
+            if stack.get("StackStatus") in ["CREATE_COMPLETE", "UPDATE_COMPLETE"]:
                 state = _poll_master_server_state(stack_name)
                 if state == "running":
-                    stack = utils.get_stack(stack_name, cfn)
                     _print_stack_outputs(stack)
-            elif stack_status in ["ROLLBACK_COMPLETE", "CREATE_FAILED", "DELETE_FAILED", "UPDATE_ROLLBACK_COMPLETE"]:
-                events = cfn.describe_stack_events(StackName=stack_name).get("StackEvents")
+            elif stack.get("StackStatus") in [
+                "ROLLBACK_COMPLETE",
+                "CREATE_FAILED",
+                "DELETE_FAILED",
+                "UPDATE_ROLLBACK_COMPLETE",
+            ]:
+                events = utils.get_stack_events(stack_name)
                 for event in events:
                     if event.get("ResourceStatus") in ["CREATE_FAILED", "DELETE_FAILED", "UPDATE_FAILED"]:
                         LOGGER.info(
@@ -635,18 +625,43 @@ def status(args):  # noqa: C901 FIXME!!!
         sys.exit(0)
 
 
-def delete(args):
-    saw_update = False
-    LOGGER.info("Deleting: %s", args.cluster_name)
-    stack_name = utils.get_stack_name(args.cluster_name)
+def _get_unretained_cw_log_group_resource_keys(template):
+    """Return the keys to all CloudWatch log group resources in template if the resource is not to be retained."""
+    unretained_cw_log_group_keys = []
+    for key, resource in template.get("Resources", {}).items():
+        if resource.get("Type") == "AWS::Logs::LogGroup" and resource.get("DeletionPolicy") != "Retain":
+            unretained_cw_log_group_keys.append(key)
+    return unretained_cw_log_group_keys
 
-    # Parse configuration file to read the AWS section
-    PclusterConfig.init_aws(config_file=args.config_file)
 
+def _persist_stack_resources(stack, template, keys):
+    """Set the resources in template identified by keys to have a DeletionPolicy of 'Retain'."""
+    for key in keys:
+        template["Resources"][key]["DeletionPolicy"] = "Retain"
+    utils.update_stack_template(stack.get("StackName"), template, stack.get("Parameters"))
+
+
+def _persist_cloudwatch_log_groups(cluster_name):
+    """Enable cluster's CloudWatch log groups to persist past cluster deletion."""
+    LOGGER.info("Configuring {0}'s CloudWatch log groups to persist past cluster deletion.".format(cluster_name))
+    substacks = utils.get_cluster_substacks(cluster_name)
+    substack_template_pairs = [(stack, utils.get_stack_template(stack.get("StackName"))) for stack in substacks]
+    substack_template_keys_triplets = [
+        (s, t, _get_unretained_cw_log_group_resource_keys(t)) for s, t in substack_template_pairs
+    ]
+    for stack, template, keys in substack_template_keys_triplets:
+        if keys:  # Only persist the CloudWatch group
+            _persist_stack_resources(stack, template, keys)
+
+
+def _delete_cluster(cluster_name, nowait):
+    """Delete cluster described by cluster_name."""
     cfn = boto3.client("cloudformation")
+    saw_update = False
     try:
         # delete_stack does not raise an exception if stack does not exist
         # Use describe_stacks to explicitly check if the stack exists
+        stack_name = utils.get_stack_name(cluster_name)
         cfn.describe_stacks(StackName=stack_name)
         cfn.delete_stack(StackName=stack_name)
         saw_update = True
@@ -654,11 +669,11 @@ def delete(args):
         sys.stdout.write("\rStatus: %s" % stack_status)
         sys.stdout.flush()
         LOGGER.debug("Status: %s", stack_status)
-        if not args.nowait:
+        if not nowait:
             while stack_status == "DELETE_IN_PROGRESS":
                 time.sleep(5)
-                stack_status = cfn.describe_stacks(StackName=stack_name).get("Stacks")[0].get("StackStatus")
-                events = cfn.describe_stack_events(StackName=stack_name).get("StackEvents")[0]
+                stack_status = utils.get_stack(stack_name, cfn, raise_on_error=True).get("StackStatus")
+                events = utils.get_stack_events(stack_name, raise_on_error=True)[0]
                 resource_status = (
                     "Status: %s - %s" % (events.get("LogicalResourceId"), events.get("ResourceStatus"))
                 ).ljust(80)
@@ -671,7 +686,7 @@ def delete(args):
             sys.stdout.write("\n")
             sys.stdout.flush()
         if stack_status == "DELETE_FAILED":
-            LOGGER.info("Cluster did not delete successfully. Run 'pcluster delete %s' again", args.cluster_name)
+            LOGGER.info("Cluster did not delete successfully. Run 'pcluster delete %s' again", cluster_name)
     except ClientError as e:
         if e.response.get("Error").get("Message").endswith("does not exist"):
             if saw_update:
@@ -685,6 +700,24 @@ def delete(args):
         sys.exit(0)
 
 
+def delete(args):
+    PclusterConfig.init_aws(config_file=args.config_file)
+    LOGGER.info("Deleting: %s", args.cluster_name)
+    stack_name = utils.get_stack_name(args.cluster_name)
+    if not utils.stack_exists(stack_name):
+        if args.keep_logs:
+            utils.warn(
+                "Stack for {0} does not exist. Cannot prevent its log groups from being deleted.".format(
+                    args.cluster_name
+                )
+            )
+        utils.warn("Cluster {0} has already been deleted.".format(args.cluster_name))
+        sys.exit(0)
+    elif args.keep_logs:
+        _persist_cloudwatch_log_groups(args.cluster_name)
+    _delete_cluster(args.cluster_name, args.nowait)
+
+
 def _get_cookbook_url(region, template_url, args, tmpdir):
     if args.custom_ami_cookbook is not None:
         return args.custom_ami_cookbook
@@ -798,7 +831,7 @@ def _run_packer(packer_command, packer_env):
         sys.stdout.flush()
         LOGGER.error("Failed to run %s\n", _command)
         sys.exit(1)
-    except (IOError, OSError):
+    except (IOError, OSError):  # noqa: B014
         sys.stdout.flush()
         LOGGER.error("Failed to run %s\nCommand not found", packer_command)
         sys.exit(1)
