@@ -12,9 +12,9 @@ from future.moves.collections import OrderedDict
 
 from pcluster.config.param_types import (
     AdditionalIamPoliciesParam,
-    AvailabilityZoneParam,
     BoolParam,
     ClusterSection,
+    ComputeAvailabilityZoneParam,
     DisableHyperThreadingParam,
     EBSSettingsParam,
     EFSSection,
@@ -23,6 +23,7 @@ from pcluster.config.param_types import (
     IntParam,
     JsonParam,
     MaintainInitialSizeParam,
+    MasterAvailabilityZoneParam,
     QueueSizeParam,
     Section,
     SettingsParam,
@@ -40,6 +41,7 @@ from pcluster.config.validators import (
     ec2_ebs_snapshot_validator,
     ec2_iam_policies_validator,
     ec2_iam_role_validator,
+    ec2_instance_type_validator,
     ec2_key_pair_validator,
     ec2_placement_group_validator,
     ec2_security_group_validator,
@@ -54,11 +56,14 @@ from pcluster.config.validators import (
     fsx_storage_capacity_validator,
     fsx_validator,
     intel_hpc_validator,
+    kms_key_validator,
     raid_volume_iops_validator,
+    s3_bucket_validator,
     scheduler_validator,
     shared_dir_validator,
     url_validator,
 )
+from pcluster.constants import CIDR_ALL_IPS
 
 # This file contains a definition of all the sections and the parameters configurable by the user
 # in the configuration file.
@@ -103,6 +108,8 @@ ALLOWED_VALUES = {
     "volume_id": r"^vol-[0-9a-z]{8}$|^vol-[0-9a-z]{17}$",
     "volume_types": ["standard", "io1", "gp2", "st1", "sc1"],
     "vpc_id": r"^vpc-[0-9a-z]{8}$|^vpc-[0-9a-z]{17}$",
+    "deployment_type": ["SCRATCH_1", "SCRATCH_2", "PERSISTENT_1"],
+    "per_unit_storage_throughput": [50, 100, 200],
 }
 
 AWS = {
@@ -175,7 +182,7 @@ VPC = {
             "validators": [ec2_subnet_id_validator],
         },
         "ssh_from": {
-            "default": "0.0.0.0/0",
+            "default": CIDR_ALL_IPS,
             "allowed_values": ALLOWED_VALUES["cidr"],
             "cfn_param_mapping": "AccessFrom",
         },
@@ -205,8 +212,12 @@ VPC = {
         },
         "master_availability_zone": {
             # NOTE: this is not exposed as a configuration parameter
-            "type": AvailabilityZoneParam,
+            "type": MasterAvailabilityZoneParam,
             "cfn_param_mapping": "AvailabilityZone",
+        },
+        "compute_availability_zone": {
+            # NOTE: this is not exposed as a configuration parameter
+            "type": ComputeAvailabilityZoneParam,
         }
     },
 }
@@ -248,6 +259,7 @@ EBS = {
         },
         "ebs_kms_key_id": {
             "cfn_param_mapping": "EBSKMSKeyId",
+            "validators": [kms_key_validator],
         },
         "ebs_volume_id": {
             "cfn_param_mapping": "EBSVolumeId",
@@ -277,7 +289,9 @@ EFS = {
                 "default": "generalPurpose",
                 "allowed_values": ["generalPurpose", "maxIO"],
             }),
-            ("efs_kms_key_id", {}),
+            ("efs_kms_key_id", {
+                "validators": [kms_key_validator],
+            }),
             ("provisioned_throughput", {
                 "allowed_values": r"^([0-9]{1,3}|10[0-1][0-9]|102[0-4])(\.[0-9])?$",  # 0.0 to 1024.0
                 "type": FloatParam,
@@ -330,7 +344,9 @@ RAID = {
                 "type": BoolParam,
                 "default": False,
             }),
-            ("ebs_kms_key_id", {}),
+            ("ebs_kms_key_id", {
+                "validators": [kms_key_validator],
+            }),
         ]
     )
 }
@@ -340,7 +356,7 @@ FSX = {
     "type": Section,
     "key": "fsx",
     "default_label": "default",
-    "validators": [fsx_validator],
+    "validators": [fsx_validator, fsx_storage_capacity_validator],
     "cfn_param_mapping": "FSXOptions",  # All the parameters in the section are converted into a single CFN parameter
     "params": OrderedDict(  # Use OrderedDict because the parameters must respect the order in the CFN parameter
         [
@@ -354,16 +370,30 @@ FSX = {
             }),
             ("storage_capacity", {
                 "type": IntParam,
-                "validators": [fsx_storage_capacity_validator]
             }),
-            ("fsx_kms_key_id", {}),
+            ("fsx_kms_key_id", {
+                "validators": [kms_key_validator],
+            }),
             ("imported_file_chunk_size", {
                 "type": IntParam,
                 "validators": [fsx_imported_file_chunk_size_validator]
             }),
-            ("export_path", {}),  # TODO add regex
-            ("import_path", {}),  # TODO add regex
-            ("weekly_maintenance_start_time", {}),  # TODO add regex
+            ("export_path", {
+                "validators": [s3_bucket_validator],
+            }),
+            ("import_path", {
+                "validators": [s3_bucket_validator],
+            }),
+            ("weekly_maintenance_start_time", {
+                "allowed_values": r"NONE|^[1-7]:([01]\d|2[0-3]):?([0-5]\d)$",
+            }),
+            ("deployment_type", {
+                "allowed_values": ALLOWED_VALUES["deployment_type"],
+            }),
+            ("per_unit_storage_throughput", {
+                "type": IntParam,
+                "allowed_values": ALLOWED_VALUES["per_unit_storage_throughput"],
+            }),
         ]
     )
 }
@@ -384,13 +414,34 @@ DCV = {
                 "default": 8443,
             }),
             ("access_from", {
-                "default": "0.0.0.0/0",
+                "default": CIDR_ALL_IPS,
                 "allowed_values": ALLOWED_VALUES["cidr"],
             }),
         ]
     )
 }
 
+CW_LOG = {
+    "type": Section,
+    "key": "cw_log",
+    "default_label": "default",
+    "cfn_param_mapping": "CWLogOptions",  # Stringify params into single CFN parameter
+    "params": OrderedDict([
+        ("enable", {
+            "type": BoolParam,
+            "default": True,
+        }),
+        ("retention_days", {
+            "type": IntParam,
+            "default": 14,
+            "cfn_param_mapping": "CWLogEventRententionDays",
+            "allowed_values": [
+                1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180, 365, 400, 545, 731, 1827, 3653
+            ]
+        }),
+    ])
+}
+
 CLUSTER = {
     "type": ClusterSection,
     "key": "cluster",
@@ -405,7 +456,7 @@ CLUSTER = {
             ("base_os", {
                 "default": "alinux",
                 "cfn_param_mapping": "BaseOS",
-                "allowed_values": ["alinux", "ubuntu1604", "ubuntu1804", "centos6", "centos7"],
+                "allowed_values": ["alinux", "alinux2", "ubuntu1604", "ubuntu1804", "centos6", "centos7"],
             }),
             ("scheduler", {
                 "default": "sge",
@@ -426,6 +477,7 @@ CLUSTER = {
             ("master_instance_type", {
                 "default": "t2.micro",
                 "cfn_param_mapping": "MasterInstanceType",
+                "validators": [ec2_instance_type_validator],
             }),
             ("master_root_volume_size", {
                 "type": IntParam,
@@ -501,16 +553,11 @@ CLUSTER = {
                 "cfn_param_mapping": "EC2IAMRoleName",
                 "validators": [ec2_iam_role_validator],  # TODO add regex
             }),
-            ("additional_iam_policies", {
-                "type": AdditionalIamPoliciesParam,
-                "cfn_param_mapping": "EC2IAMPolicies",
-                "validators": [ec2_iam_policies_validator],
-            }),
             ("s3_read_resource", {
-                "cfn_param_mapping": "S3ReadResource",  # TODO add validator
+                "cfn_param_mapping": "S3ReadResource",
             }),
             ("s3_read_write_resource", {
-                "cfn_param_mapping": "S3ReadWriteResource",  # TODO add validator
+                "cfn_param_mapping": "S3ReadWriteResource",
             }),
             (
                 "disable_hyperthreading",
@@ -629,6 +676,17 @@ CLUSTER = {
                 "type": SettingsParam,
                 "referred_section": DCV,
             }),
+            ("cw_log_settings", {
+                "type": SettingsParam,
+                "referred_section": CW_LOG,
+            }),
+            # Moved from the "Access and Networking" section because its configuration is
+            # dependent on multiple other parameters from within this section.
+            ("additional_iam_policies", {
+                "type": AdditionalIamPoliciesParam,
+                "cfn_param_mapping": "EC2IAMPolicies",
+                "validators": [ec2_iam_policies_validator],
+            }),
         ]
     )
 }
