@@ -8,6 +8,7 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
+import logging
 import re
 import urllib.error
 import urllib.request
@@ -16,15 +17,28 @@ from urllib.parse import urlparse
 import boto3
 from botocore.exceptions import ClientError
 
+from pcluster.constants import CIDR_ALL_IPS
 from pcluster.dcv.utils import get_supported_dcv_os, get_supported_dcv_partition
 from pcluster.utils import (
     get_efs_mount_target_id,
     get_instance_vcpus,
     get_partition,
     get_region,
+    get_supported_compute_instance_types,
     get_supported_features,
+    get_supported_instance_types,
+    get_supported_os,
 )
 
+LOGFILE_LOGGER = logging.getLogger("cli_log_file")
+
+DCV_MESSAGES = {
+    "warnings": {
+        "access_from_world": "With this configuration you are opening dcv port ({port}) to the world (0.0.0.0/0). "
+        "It is recommended to use dcv access_from config option to restrict access."
+    }
+}
+
 
 def _get_sts_endpoint():
     """Get regionalized STS endpoint."""
@@ -133,6 +147,12 @@ def fsx_validator(section_key, section_label, pcluster_config):
     if fsx_export_path and not fsx_import_path:
         errors.append("When specifying 'export_path', the 'import_path' option must be specified")
 
+    if fsx_section.get_param_value("deployment_type") != "PERSISTENT_1":
+        if fsx_section.get_param_value("fsx_kms_key_id"):
+            errors.append("'fsx_kms_key_id' can only be used when 'deployment_type = PERSISTENT_1'")
+        if fsx_section.get_param_value("per_unit_storage_throughput"):
+            errors.append("'per_unit_storage_throughput' can only be used when 'deployment_type = PERSISTENT_1'")
+
     return errors, warnings
 
 
@@ -188,14 +208,28 @@ def fsx_id_validator(param_key, param_value, pcluster_config):
     return errors, warnings
 
 
-def fsx_storage_capacity_validator(param_key, param_value, pcluster_config):
+def fsx_storage_capacity_validator(section_key, section_label, pcluster_config):
     errors = []
     warnings = []
 
-    if int(param_value) > 0 and not (
-        int(param_value) == 1200 or int(param_value) == 2400 or int(param_value) % 3600 == 0
-    ):
-        errors.append("Capacity for FSx lustre filesystem, 1,200 GB, 2,400 GB or increments of 3,600 GB")
+    fsx_section = pcluster_config.get_section(section_key, section_label)
+    storage_capacity = fsx_section.get_param_value("storage_capacity")
+    deployment_type = fsx_section.get_param_value("deployment_type")
+
+    if fsx_section.get_param_value("fsx_fs_id"):
+        # if fsx_fs_id is provided, don't validate storage_capacity
+        return errors, warnings
+    elif not storage_capacity:
+        # if fsx_fs_id is not provided, storage_capacity must be provided
+        errors.append("When specifying 'fsx' section, the 'storage_capacity' option must be specified")
+    elif deployment_type == "SCRATCH_1":
+        if not (storage_capacity == 1200 or storage_capacity == 2400 or storage_capacity % 3600 == 0):
+            warnings.append("Capacity for FSx SCRATCH_1 filesystem is 1,200 GB, 2,400 GB or increments of 3,600 GB")
+    elif deployment_type in ["SCRATCH_2", "PERSISTENT_1"]:
+        if not (storage_capacity == 1200 or storage_capacity % 2400 == 0):
+            warnings.append(
+                "Capacity for FSx SCRATCH_2 and PERSISTENT_1 filesystems is 1,200 GB or increments of 2,400 GB"
+            )
 
     return errors, warnings
 
@@ -231,6 +265,13 @@ def dcv_enabled_validator(param_key, param_value, pcluster_config):
         if get_partition() not in get_supported_dcv_partition():
             errors.append("NICE DCV is not supported in the selected region '{0}'".format(get_region()))
 
+        if pcluster_config.get_section("dcv").get_param_value("access_from") == CIDR_ALL_IPS:
+            LOGFILE_LOGGER.warning(
+                DCV_MESSAGES["warnings"]["access_from_world"].format(
+                    port=pcluster_config.get_section("dcv").get_param_value("port")
+                )
+            )
+
     return errors, warnings
 
 
@@ -244,6 +285,18 @@ def fsx_imported_file_chunk_size_validator(param_key, param_value, pcluster_conf
     return errors, warnings
 
 
+def kms_key_validator(param_key, param_value, pcluster_config):
+    errors = []
+    warnings = []
+
+    try:
+        boto3.client("kms").describe_key(KeyId=param_value)
+    except ClientError as e:
+        errors.append(e.response.get("Error").get("Message"))
+
+    return errors, warnings
+
+
 def efa_validator(param_key, param_value, pcluster_config):
     errors = []
     warnings = []
@@ -257,7 +310,7 @@ def efa_validator(param_key, param_value, pcluster_config):
             "to one of the following values : {1}".format(param_value, allowed_instances)
         )
 
-    allowed_oses = ["alinux", "centos7", "ubuntu1604", "ubuntu1804"]
+    allowed_oses = ["alinux", "alinux2", "centos7", "ubuntu1604", "ubuntu1804"]
     if cluster_section.get_param_value("base_os") not in allowed_oses:
         errors.append(
             "When using 'enable_efa = {0}' it is required to set the 'base_os' parameter "
@@ -420,6 +473,19 @@ def _get_pcluster_user_policy(partition, region, account_id):
     ]
 
 
+def ec2_instance_type_validator(param_key, param_value, pcluster_config):
+    errors = []
+    warnings = []
+
+    if param_value not in get_supported_instance_types():
+        errors.append(
+            "The instance type '{0}' used for the '{1}' parameter is not supported by AWS ParallelCluster.".format(
+                param_value, param_key
+            )
+        )
+    return errors, warnings
+
+
 def ec2_vpc_id_validator(param_key, param_value, pcluster_config):
     errors = []
     warnings = []
@@ -523,6 +589,27 @@ def url_validator(param_key, param_value, pcluster_config):
     return errors, warnings
 
 
+def s3_bucket_validator(param_key, param_value, pcluster_config):
+    errors = []
+    warnings = []
+
+    if urlparse(param_value).scheme == "s3":
+        try:
+            bucket = param_value.split("/")[2]
+            boto3.client("s3").head_bucket(Bucket=bucket)
+        except ClientError:
+            warnings.append(
+                "The S3 bucket '{0}' does not exist or you do not have access to it. "
+                "Please be sure the cluster nodes have access to it.".format(param_value)
+            )
+    else:
+        errors.append(
+            "The value '{0}' used for the parameter '{1}' is not a valid S3 URI.".format(param_value, param_key)
+        )
+
+    return errors, warnings
+
+
 def ec2_ebs_snapshot_validator(param_key, param_value, pcluster_config):
     errors = []
     warnings = []
@@ -634,6 +721,10 @@ def scheduler_validator(param_key, param_value, pcluster_config):
         if pcluster_config.region in ["ap-northeast-3", "us-gov-east-1", "us-gov-west-1"]:
             errors.append("'awsbatch' scheduler is not supported in the '{0}' region".format(pcluster_config.region))
 
+    supported_os = get_supported_os(param_value)
+    if pcluster_config.get_section("cluster").get_param_value("base_os") not in supported_os:
+        errors.append("'{0}' scheduler supports the following Operating Systems: {1}".format(param_value, supported_os))
+
     return errors, warnings
 
 
@@ -672,46 +763,44 @@ def cluster_validator(section_key, section_label, pcluster_config):
 
 
 def compute_instance_type_validator(param_key, param_value, pcluster_config):
+    """Validate compute instance type, calling ec2_instance_type_validator if the scheduler is not awsbatch."""
     errors = []
     warnings = []
 
     cluster_config = pcluster_config.get_section("cluster")
-    if cluster_config.get_param_value("scheduler") == "awsbatch":
-
-        try:
-            supported_instances = get_supported_features(pcluster_config.region, "batch").get("instances")
-            if supported_instances:
-                for instance in param_value.split(","):
-                    if not instance.strip() in supported_instances:
-                        errors.append(
-                            "compute_instance_type '{0}' is not supported by awsbatch in region '{1}'".format(
-                                instance, pcluster_config.region
-                            )
+    scheduler = cluster_config.get_param_value("scheduler")
+    if scheduler == "awsbatch":
+        supported_instances = get_supported_compute_instance_types(scheduler)
+        if supported_instances:
+            for instance in param_value.split(","):
+                if not instance.strip() in supported_instances:
+                    errors.append(
+                        "compute_instance_type '{0}' is not supported by awsbatch in region '{1}'".format(
+                            instance, pcluster_config.region
                         )
-            else:
-                warnings.append(
-                    "Unable to get instance types supported by awsbatch. Skipping compute_instance_type validation"
-                )
-
-            if "," not in param_value and "." in param_value:
-                # if the type is not a list, and contains dot (nor optimal, nor a family)
-                # validate instance type against max_vcpus limit
-                vcpus = get_instance_vcpus(pcluster_config.region, param_value)
-                if vcpus <= 0:
-                    warnings.append(
-                        "Unable to get the number of vcpus for the compute_instance_type '{0}'. "
-                        "Skipping instance type against max_vcpus validation".format(param_value)
                     )
-                else:
-                    if cluster_config.get_param_value("max_vcpus") < vcpus:
-                        errors.append(
-                            "max_vcpus must be greater than or equal to {0}, that is the number of vcpus "
-                            "available for the {1} that you selected as compute_instance_type".format(
-                                vcpus, param_value
-                            )
-                        )
-        except ClientError as e:
-            errors.append(e.response.get("Error").get("Message"))
+        else:
+            warnings.append(
+                "Unable to get instance types supported by awsbatch. Skipping compute_instance_type validation"
+            )
+
+        if "," not in param_value and "." in param_value:
+            # if the type is not a list, and contains dot (nor optimal, nor a family)
+            # validate instance type against max_vcpus limit
+            vcpus = get_instance_vcpus(pcluster_config.region, param_value)
+            if vcpus <= 0:
+                warnings.append(
+                    "Unable to get the number of vcpus for the compute_instance_type '{0}'. "
+                    "Skipping instance type against max_vcpus validation".format(param_value)
+                )
+            else:
+                if cluster_config.get_param_value("max_vcpus") < vcpus:
+                    errors.append(
+                        "max_vcpus must be greater than or equal to {0}, that is the number of vcpus "
+                        "available for the {1} that you selected as compute_instance_type".format(vcpus, param_value)
+                    )
+    else:
+        errors, warnings = ec2_instance_type_validator(param_key, param_value, pcluster_config)
 
     return errors, warnings
 
