@@ -17,6 +17,7 @@ standard_library.install_aliases()
 import json
 import logging
 import os
+import re
 import sys
 import time
 import urllib.request
@@ -27,16 +28,58 @@ import boto3
 import pkg_resources
 from botocore.exceptions import ClientError
 
+from pcluster.constants import PCLUSTER_ISSUES_LINK, PCLUSTER_STACK_PREFIX
+
 LOGGER = logging.getLogger(__name__)
 
-PCLUSTER_STACK_PREFIX = "parallelcluster-"
-PCLUSTER_ISSUES_LINK = "https://github.com/aws/aws-parallelcluster/issues"
+STACK_TYPE = "AWS::CloudFormation::Stack"
 
 
 def get_stack_name(cluster_name):
     return PCLUSTER_STACK_PREFIX + cluster_name
 
 
+def get_stack_template(stack_name):
+    """Get the template used for the given stack."""
+    try:
+        template = boto3.client("cloudformation").get_template(StackName=stack_name).get("TemplateBody")
+    except ClientError as client_err:
+        error(
+            "Unable to get template for stack {stack_name}.\n{err_msg}".format(
+                stack_name=stack_name, err_msg=client_err.response.get("Error").get("Message")
+            )
+        )
+    if not template:
+        error("Unable to get template for stack {0}.".format(stack_name))
+    return template
+
+
+def _wait_for_update(stack_name):
+    """Wait for the given stack to be finished updating."""
+    while get_stack(stack_name).get("StackStatus") == "UPDATE_IN_PROGRESS":
+        time.sleep(5)
+
+
+def update_stack_template(stack_name, updated_template, cfn_parameters):
+    """Update stack_name's template to that represented by updated_template."""
+    try:
+        boto3.client("cloudformation").update_stack(
+            StackName=stack_name,
+            TemplateBody=json.dumps(updated_template, indent=2),  # Indent so it looks nice in the console
+            Parameters=cfn_parameters,
+            Capabilities=["CAPABILITY_IAM"],
+        )
+        _wait_for_update(stack_name)
+    except ClientError as client_err:
+        if "no updates are to be performed" in client_err.response.get("Error").get("Message").lower():
+            return  # If updated_template was the same as the stack's current one, consider the update a success
+        error(
+            "Unable to update stack template for stack {stack_name}: {emsg}".format(
+                stack_name=stack_name, emsg=client_err.response.get("Error").get("Message")
+            )
+        )
+
+
 def get_region():
     """Get AWS_DEFAULT_REGION from the environment."""
     return os.environ.get("AWS_DEFAULT_REGION")
@@ -202,14 +245,45 @@ def get_instance_vcpus(region, instance_type):
     return vcpus
 
 
+def get_supported_instance_types():
+    """
+    Get supported instance types.
+
+    :return: the list of supported instance types
+    """
+    try:
+        instances = _get_json_from_s3(get_region(), "instances/instances.json")
+        return instances.keys()
+    except (ValueError, ClientError):
+        error("Unable to retrieve the list of supported instance types.")
+
+
+def get_supported_compute_instance_types(scheduler):
+    """
+    Get supported instance types (and families in awsbatch case).
+
+    :param scheduler: the scheduler for which we want to know the supported compute instance types or families
+    :return: the list of supported instance types and families
+    """
+    instances = (
+        get_supported_features(get_region(), "batch").get("instances")
+        if scheduler == "awsbatch"
+        else get_supported_instance_types()
+    )
+    return instances
+
+
 def get_supported_os(scheduler):
     """
-    Return a tuple of the os supported by parallelcluster for the specific scheduler.
+    Return an array containing the list of OSes supported by parallelcluster for the specific scheduler.
 
     :param scheduler: the scheduler for which we want to know the supported os
-    :return: a tuple of strings of the supported os
+    :return: an array of strings of the supported OSes
     """
-    return "alinux" if scheduler == "awsbatch" else "alinux", "centos6", "centos7", "ubuntu1604", "ubuntu1804"
+    oses = ["alinux", "alinux2"]
+    if scheduler != "awsbatch":
+        oses.extend(["centos6", "centos7", "ubuntu1604", "ubuntu1804"])
+    return list(oses)
 
 
 def get_supported_schedulers():
@@ -232,7 +306,7 @@ def get_stack_output_value(stack_outputs, output_key):
     return next((o.get("OutputValue") for o in stack_outputs if o.get("OutputKey") == output_key), None)
 
 
-def get_stack(stack_name, cfn_client=None):
+def get_stack(stack_name, cfn_client=None, raise_on_error=False):
     """
     Get the output for a DescribeStacks action for the given Stack.
 
@@ -243,11 +317,59 @@ def get_stack(stack_name, cfn_client=None):
     try:
         if not cfn_client:
             cfn_client = boto3.client("cloudformation")
-        return cfn_client.describe_stacks(StackName=stack_name).get("Stacks")[0]
-    except (ClientError, IndexError) as e:
+        return retry_on_boto3_throttling(cfn_client.describe_stacks, StackName=stack_name).get("Stacks")[0]
+    except ClientError as e:
+        if raise_on_error:
+            raise
         error(e.response.get("Error").get("Message"))
 
 
+def stack_exists(stack_name):
+    """Return a boolean describing whether or not a stack by the given name exists."""
+    try:
+        get_stack(stack_name)
+        return True
+    except SystemExit as sys_exit:
+        if "Stack with id {0} does not exist".format(stack_name) in str(sys_exit):
+            return False
+        raise
+
+
+def get_stack_resources(stack_name):
+    """Get the given stack's resources."""
+    cfn_client = boto3.client("cloudformation")
+    try:
+        return retry_on_boto3_throttling(cfn_client.describe_stack_resources, StackName=stack_name).get(
+            "StackResources"
+        )
+    except ClientError as client_err:
+        error(
+            "Unable to get {stack_name}'s resources: {reason}".format(
+                stack_name=stack_name, reason=client_err.response.get("Error").get("Message")
+            )
+        )
+
+
+def get_stack_events(stack_name, raise_on_error=False):
+    cfn_client = boto3.client("cloudformation")
+    try:
+        return retry_on_boto3_throttling(cfn_client.describe_stack_events, StackName=stack_name).get("StackEvents")
+    except ClientError as client_err:
+        if raise_on_error:
+            raise
+        error(
+            "Unable to get {stack_name}'s events: {reason}".format(
+                stack_name=stack_name, reason=client_err.response.get("Error").get("Message")
+            )
+        )
+
+
+def get_cluster_substacks(cluster_name):
+    """Return stack objects with names that match the given prefix."""
+    resources = get_stack_resources(get_stack_name(cluster_name))
+    return [get_stack(r.get("PhysicalResourceId")) for r in resources if r.get("ResourceType") == STACK_TYPE]
+
+
 def verify_stack_creation(stack_name, cfn_client):
     """
     Wait for the stack creation to be completed and notify if the stack creation fails.
@@ -260,7 +382,7 @@ def verify_stack_creation(stack_name, cfn_client):
     resource_status = ""
     while status == "CREATE_IN_PROGRESS":
         status = get_stack(stack_name, cfn_client).get("StackStatus")
-        events = cfn_client.describe_stack_events(StackName=stack_name).get("StackEvents")[0]
+        events = get_stack_events(stack_name, raise_on_error=True)[0]
         resource_status = ("Status: %s - %s" % (events.get("LogicalResourceId"), events.get("ResourceStatus"))).ljust(
             80
         )
@@ -272,19 +394,42 @@ def verify_stack_creation(stack_name, cfn_client):
         LOGGER.debug(resource_status)
     if status != "CREATE_COMPLETE":
         LOGGER.critical("\nCluster creation failed.  Failed events:")
-        events = cfn_client.describe_stack_events(StackName=stack_name).get("StackEvents")
-        for event in events:
-            if event.get("ResourceStatus") == "CREATE_FAILED":
-                LOGGER.info(
-                    "  - %s %s %s",
-                    event.get("ResourceType"),
-                    event.get("LogicalResourceId"),
-                    event.get("ResourceStatusReason"),
-                )
+        _log_stack_failure_recursive(stack_name)
         return False
     return True
 
 
+def _log_stack_failure_recursive(stack_name, indent=2):
+    """Log stack failures in recursive manner, until there is no substack layer."""
+    events = get_stack_events(stack_name, raise_on_error=True)
+    for event in events:
+        if event.get("ResourceStatus") == "CREATE_FAILED":
+            _log_failed_cfn_event(event, indent)
+            if event.get("ResourceType") == "AWS::CloudFormation::Stack":
+                # Sample substack error:
+                # "Embedded stack arn:aws:cloudformation:us-east-2:704743599507:stack/
+                # parallelcluster-fsx-fail-FSXSubstack-65ITLJEZJ0DQ/
+                # 3a4ecf00-51e7-11ea-8e3e-022fd555c652 was not successfully created:
+                # The following resource(s) failed to create: [FileSystem]."
+                substack_error = re.search(
+                    ".+/({0}.+)/".format(PCLUSTER_STACK_PREFIX), event.get("ResourceStatusReason")
+                )
+                substack_name = substack_error.group(1) if substack_error else None
+                if substack_name:
+                    _log_stack_failure_recursive(substack_name, indent=indent + 2)
+
+
+def _log_failed_cfn_event(event, indent):
+    """Log failed CFN events."""
+    LOGGER.info(
+        "%s- %s %s %s",
+        " " * indent,
+        event.get("ResourceType"),
+        event.get("LogicalResourceId"),
+        event.get("ResourceStatusReason"),
+    )
+
+
 def get_templates_bucket_path():
     """Return a string containing the path of bucket."""
     region = get_region()
@@ -372,26 +517,6 @@ def get_avail_zone(subnet_id):
     return avail_zone
 
 
-def get_latest_alinux_ami_id():
-    """Get latest alinux ami id."""
-    try:
-        alinux_ami_id = (
-            boto3.client("ssm")
-            .get_parameters_by_path(Path="/aws/service/ami-amazon-linux-latest")
-            .get("Parameters")[0]
-            .get("Value")
-        )
-    except ClientError as e:
-        error("Unable to retrieve Amazon Linux AMI id.\n{0}".format(e.response.get("Error").get("Message")))
-
-    return alinux_ami_id
-
-
-def list_ec2_instance_types():
-    """Return a list of all the instance types available on EC2, independent by the region."""
-    return boto3.client("ec2").meta.service_model.shape_for("InstanceType").enum
-
-
 def get_master_server_id(stack_name):
     """Return the physical id of the master server, or [] if no master server."""
     try:
@@ -485,3 +610,14 @@ def retry(func, func_args, attempts=1, wait=0):
 
             LOGGER.debug("{0}, retrying in {1} seconds..".format(e, wait))
             time.sleep(wait)
+
+
+def retry_on_boto3_throttling(func, wait=5, *args, **kwargs):
+    while True:
+        try:
+            return func(*args, **kwargs)
+        except ClientError as e:
+            if e.response["Error"]["Code"] != "Throttling":
+                raise
+            LOGGER.debug("Throttling when calling %s function. Will retry in %d seconds.", func.__name__, wait)
+            time.sleep(wait)
